settings_updater = {
    # larch processing
    "est.core.process.larch.autobk.Larch_autobk": "set_autobk_params",
    "est.core.process.larch.mback.Larch_mback": "set_mback_params",
    "est.core.process.larch.mback_norm.Larch_mback_norm": "set_mback_norm_params",
    "est.core.process.larch.pre_edge.Larch_pre_edge": "set_pre_edge_params",
    "est.core.process.larch.xftf.Larch_xftf": "set_xftf_params",
    # utils
    "est.core.process.io.DumpXasObject": "set_output_params",
    "est.core.process.noise.NoiseProcess": "set_noise_params",
    "est.core.process.energyroi.EnergyROIProcess": "set_energyroi_params",
}
