aliases = {
    # define aliases for larch
    "orangecontrib.est.widgets.larch.autobk.AutobkOW": "est.core.process.larch.autobk.Larch_autobk",
    "orangecontrib.est.widgets.larch.mback.MbackOW": "est.core.process.larch.mback.Larch_mback",
    "orangecontrib.est.widgets.larch.mback_norm.Mback_normOW": "est.core.process.larch.mback_norm.Larch_mback_norm",
    "orangecontrib.est.widgets.larch.pre_edge.PreEdgeOW": "est.core.process.larch.pre_edge.Larch_pre_edge",
    "orangecontrib.est.widgets.larch.xftf.XFTFOW": "est.core.process.larch.xftf.Larch_xftf",
    # define aliases for pymca
    "orangecontrib.est.widgets.pymca.exafs.ExafsOW": "est.core.process.pymca.exafs.PyMca_exafs",
    "orangecontrib.est.widgets.pymca.ft.FTOW": "est.core.process.pymca.ft.PyMca_ft",
    "orangecontrib.est.widgets.pymca.k_weight.KWeightOW": "est.core.process.pymca.k_weight.PyMca_k_weight",
    "orangecontrib.est.widgets.pymca.normalization.NormalizationOW": "est.core.process.pymca.normalization.PyMca_normalization",
    # define aliases for utils
    "orangecontrib.est.widgets.utils.roi.RoiSelectionOW": "est.core.process.roi.ROIProcess",
    "orangecontrib.est.widgets.utils.converter.ConverterOW": "est.core.process.reconstruction.axis.AxisProcess",
    "orangecontrib.est.widgets.utils.e0calculator.E0calculatorOW": "pypushflow.utils.IgnoreProcess",
    "orangecontrib.est.widgets.utils.noise.NoiseOW": "est.core.process.noise.NoiseProcess",
    "orangecontrib.est.widgets.utils.saving_points.SavingPointOW": "pypushflow.utils.IgnoreProcess",
    "orangecontrib.est.widgets.utils.xas_input.XASInputOW": "pypushflow.utils.IgnoreProcess",
    "orangecontrib.est.widgets.utils.xas_output.XASOutputOW": "est.core.process.io.DumpXasObject",
    "orangecontrib.est.widgets.utils.energyroi.EnergyRoiOW": "est.core.process.energyroi.EnergyROIProcess",
}
